bl_info = { 
    "name": "Show Collection/Object/Data Users of the Selected Object in the Viewport",
    "author": "1P2D",
    "version": (1, 0, 4),
    "blender": (4, 2, 0),
    "location": "View3D > Object > Viewport Display",
    "description": "Display the number of collection, object, and data users in the 3D viewport",
    "category": "Object Properties > Viewport Display > Show",
}

import bpy
import blf
from bpy_extras.view3d_utils import location_3d_to_region_2d

handle = None

def register():
    global handle
    # Define boolean properties
    bpy.types.Object.show_data_name = bpy.props.BoolProperty(
        name="Data Name",
        description="Display the name of the data block",
        default=False  # Set to False by default
    )
    bpy.types.Object.show_data_users = bpy.props.BoolProperty(
        name="Data Users (Objects)",
        description="Display the number of objects sharing the data",
        default=False
    )
    bpy.types.Object.show_data_user_names = bpy.props.BoolProperty(
        name="Data User Names",
        description="Display the names of the objects sharing the data",
        default=False  # Set to False by default
    )
    bpy.types.Object.show_object_collection_users = bpy.props.BoolProperty(
        name="Object and Collection Users (Scenes)",
        description="Display the number of scenes sharing the object or its collection",
        default=False
    )
    bpy.types.Object.show_object_collection_user_names = bpy.props.BoolProperty(
        name="Object and Collection User Names",
        description="Display the names of the scenes sharing the object or its collection",
        default=False  # Set to False by default
    )

    # Append the UI panel function to the OBJECT_PT_display panel
    bpy.types.OBJECT_PT_display.append(draw_users_properties)

    # Add the draw handler if not already added
    if handle is None:
        handle = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, (), 'WINDOW', 'POST_PIXEL')

def unregister():
    global handle
    # Remove the boolean properties
    del bpy.types.Object.show_data_name
    del bpy.types.Object.show_data_users
    del bpy.types.Object.show_data_user_names
    del bpy.types.Object.show_object_collection_users
    del bpy.types.Object.show_object_collection_user_names

    # Remove the UI panel function from OBJECT_PT_display
    bpy.types.OBJECT_PT_display.remove(draw_users_properties)

    # Remove the draw handler
    if handle is not None:
        bpy.types.SpaceView3D.draw_handler_remove(handle, 'WINDOW')
        handle = None

def draw_users_properties(self, context):
    layout = self.layout
    obj = context.object
    if obj is None:
        return

    # Display each property inline with the "Show" category style
    layout.prop(obj, "show_data_name", text="Data Name")
    layout.prop(obj, "show_data_users", text="Data Users (Objects)")
    layout.prop(obj, "show_data_user_names", text="Data User Names")

    # Add a separator between Data Group and Object/Collection Users Group
    layout.separator()

    # Object/Collection Users Group
    layout.prop(obj, "show_object_collection_users", text="Object/Collection Users (Scenes)")
    layout.prop(obj, "show_object_collection_user_names", text="Object/Collection User Names")




def is_scene_collection(collection):
    """Check if the collection is the master collection of any scene."""
    for scene in bpy.data.scenes:
        if collection == scene.collection:
            return True
    return False

def is_collection_in_scene(collection, scene):
    """Recursively check if the collection is part of the scene's collections."""
    def recursive_search(col):
        if col == collection:
            return True
        for child in col.children:
            if recursive_search(child):
                return True
        return False
    for child in scene.collection.children:
        if recursive_search(child):
            return True
    return False

def count_scenes_with_collection(collection):
    """Count how many scenes include the given collection."""
    count = 0
    for scene in bpy.data.scenes:
        if is_collection_in_scene(collection, scene):
            count += 1
    return count

def get_linked_collection_info(obj):
    """Retrieve information about collections linking the object across scenes."""
    linked_info = []
    linked_via_collection = False
    seen_collections = set()

    for collection in obj.users_collection:
        if is_scene_collection(collection):
            continue  # Skip the Scene Collection
        linked_via_collection = True
        if collection.name not in seen_collections:
            scene_count = count_scenes_with_collection(collection)
            linked_info.append((collection.name, scene_count))
            seen_collections.add(collection.name)

    return linked_info, linked_via_collection

def get_object_user_scenes(obj):
    """Retrieve a list of scene names where the object is used."""
    scenes_with_object = []
    for scene in bpy.data.scenes:
        # Check if the object is directly in the scene's objects
        if obj.name in scene.objects.keys():
            scenes_with_object.append(scene.name)
            continue
        # Check if the object is in any collection within the scene
        for collection in scene.collection.children_recursive:
            if obj.name in collection.objects.keys():
                scenes_with_object.append(scene.name)
                break
    return scenes_with_object

def get_data_user_names(obj):
    """Retrieve a list of object names sharing the same data, including the object itself."""
    data_users = [obj.name]  # Start with the object itself
    for other_obj in bpy.data.objects:
        if other_obj.data == obj.data and other_obj != obj:
            data_users.append(other_obj.name)
    return data_users

def get_collection_user_names(obj, linked_collections):
    """Retrieve a dictionary of scene names sharing the linked collections."""
    collection_user_names = {}
    for collection_name, _ in linked_collections:
        scene_names = []
        collection = bpy.data.collections.get(collection_name)
        if collection:
            for scene in bpy.data.scenes:
                if is_collection_in_scene(collection, scene):
                    scene_names.append(scene.name)
        collection_user_names[collection_name] = scene_names
    return collection_user_names

def draw_callback_px():
    context = bpy.context

    if not isinstance(context.space_data, bpy.types.SpaceView3D):
        return

    if not context.space_data.overlay.show_overlays:
        return

    font_id = 0
    blf.size(font_id, 11)  # Fixed: only two arguments
    theme = context.preferences.themes[0]
    view_3d_theme = theme.view_3d

    # Get the necessary colors from the theme
    wire_color = (*view_3d_theme.wire, 1.0)
    object_active_color = (*view_3d_theme.object_active, 1.0)
    object_selected_color = (*view_3d_theme.object_selected, 1.0)

    x_offset = 10
    group_spacing = 10  # Pixels between groups
    line_spacing = 3    # Pixels between lines

    window_region = None
    area = None
    for area in context.screen.areas:
        if area.type == 'VIEW_3D':
            for region in area.regions:
                if region.type == 'WINDOW':
                    window_region = region
                    break
            if window_region:
                break
    if not window_region:
        return

    rv3d = area.spaces.active.region_3d

    for obj in context.visible_objects:
        try:
            # Check if any of the toggles are enabled for the object
            if (obj.show_data_users or obj.show_data_user_names or
                obj.show_data_name or
                obj.show_object_collection_users or obj.show_object_collection_user_names):
                obj_location = obj.matrix_world.translation
                coord_2d = location_3d_to_region_2d(window_region, rv3d, obj_location)

                if coord_2d:
                    # Determine the color based on the object's state
                    if obj.mode == 'EDIT':
                        color = wire_color
                    elif obj == context.view_layer.objects.active and obj.select_get():
                        color = object_active_color
                    elif obj == context.view_layer.objects.active and not obj.select_get():
                        color = wire_color
                    elif obj.select_get():
                        color = object_selected_color
                    else:
                        color = wire_color

                    blf.color(font_id, *color)

                    # Start drawing below the object name (assuming Blender's built-in name is already displayed)
                    y_offset = coord_2d.y - 20  # Adjust as needed to position below the name

                    # Data Group
                    if obj.show_data_name or obj.show_data_users or obj.show_data_user_names:
                        # Display Data Name if enabled
                        if obj.show_data_name:
                            data_name_text = f"Data Name: {obj.data.name}" if obj.data else "Data Name: N/A"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_name_text)
                            _, data_name_height = blf.dimensions(font_id, data_name_text)
                            y_offset -= (data_name_height + line_spacing)

                        # Display Data Users count if enabled
                        if obj.show_data_users:
                            data_users_text = f"Data Users (Objects): {obj.data.users}" if obj.data else "Data Users (Objects): N/A"
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_users_text)
                            _, data_users_height = blf.dimensions(font_id, data_users_text)
                            y_offset -= (data_users_height + line_spacing)

                        # Display Data User Names if enabled
                        if obj.show_data_user_names:
                            if obj.data:
                                # Retrieve data user names if data exists
                                data_user_objects = get_data_user_names(obj)
                                if data_user_objects:
                                    data_users_str = ", ".join(data_user_objects)
                                    data_user_names_text = f"Data User Names: {data_users_str}"
                                else:
                                    data_user_names_text = "Data User Names: None"
                            else:
                                # Set to N/A if data is not accessible
                                data_user_names_text = "Data User Names: N/A"
                            
                            blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                            blf.draw(font_id, data_user_names_text)
                            _, data_user_names_height = blf.dimensions(font_id, data_user_names_text)
                            y_offset -= (data_user_names_height + group_spacing)
                        else:
                            # If Data User Names are not shown but other Data toggles are, add group spacing
                            if obj.show_data_name or obj.show_data_users:
                                y_offset -= group_spacing

                    # Object/Collection Users Group
                    if obj.show_object_collection_users or obj.show_object_collection_user_names:
                        linked_collections, linked_via_collection = get_linked_collection_info(obj)
                        if obj.show_object_collection_users:
                            if linked_via_collection and linked_collections:
                                collection_user_names = get_collection_user_names(obj, linked_collections)
                                for collection_name, scene_count in linked_collections:
                                    # Object Users via Collection "CollectionName"(Scenes): Y
                                    object_users_text = f"Object Users (Scenes) via collection \"{collection_name}\": {scene_count}"
                                    blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                    blf.draw(font_id, object_users_text)
                                    _, object_users_height = blf.dimensions(font_id, object_users_text)
                                    y_offset -= (object_users_height + line_spacing)

                                    # Scene Names: A, B, C
                                    if obj.show_object_collection_user_names:
                                        scenes = collection_user_names.get(collection_name, [])
                                        if scenes:
                                            scenes_str = ", ".join(scenes)
                                            collection_user_names_text = f"Object User Names: {scenes_str}"
                                        else:
                                            collection_user_names_text = "Object User: None"
                                        blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                        blf.draw(font_id, collection_user_names_text)
                                        _, collection_user_names_height = blf.dimensions(font_id, collection_user_names_text)
                                        y_offset -= (collection_user_names_height + group_spacing)
                                    else:
                                        # If Collection User Names are not shown, add group spacing
                                        y_offset -= group_spacing
                            else:
                                # Object Users (Scenes): X (Direct Sharing)
                                object_users_count = obj.users
                                object_users_text = f"Object Users (Scenes): {object_users_count}"
                                blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                blf.draw(font_id, object_users_text)
                                _, object_users_height = blf.dimensions(font_id, object_users_text)
                                y_offset -= (object_users_height + line_spacing)

                                # Scene Names: A, B, C
                                if obj.show_object_collection_user_names:
                                    scenes = get_object_user_scenes(obj)
                                    if len(scenes) > 0:
                                        scenes_str = ", ".join(scenes)
                                        object_user_scenes_text = f"Object User Names: {scenes_str}"
                                    else:
                                        object_user_scenes_text = "Object User: None"
                                    blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                    blf.draw(font_id, object_user_scenes_text)
                                    _, object_user_scenes_height = blf.dimensions(font_id, object_user_scenes_text)
                                    y_offset -= (object_user_scenes_height + group_spacing)
                                else:
                                    # If Object User Names are not shown, add group spacing
                                    y_offset -= group_spacing
                        else:
                            # If Object/Collection Users are not enabled but User Names are
                            if obj.show_object_collection_user_names:
                                # Handle cases where user names are shown without showing user counts
                                # This scenario might be less common but is handled for completeness
                                scenes = get_object_user_scenes(obj)
                                if len(scenes) > 0:
                                    scenes_str = ", ".join(scenes)
                                    object_user_scenes_text = f"Object User Names: {scenes_str}"
                                else:
                                    object_user_scenes_text = "Object User: None"
                                blf.position(font_id, coord_2d.x + x_offset, y_offset, 0)
                                blf.draw(font_id, object_user_scenes_text)
                                _, object_user_scenes_height = blf.dimensions(font_id, object_user_scenes_text)
                                y_offset -= (object_user_scenes_height + group_spacing)

        except Exception as e:
            print(f"Error processing object '{obj.name}': {e}")

if __name__ == "__main__":
    register()
